{% extends "./post.html" %}

{% block meta %}
  {% if episode.podcast_audio %}
    <!-- Twitter Player Card / Open Graph Tags-->
    <meta name="twitter:card" content="player">
    <meta name="twitter:title" content="{{ page.seo_title }}">
    <meta name="twitter:description" content="{{ page.search_description }}">
    <meta name="twitter:image" content="{{ cover_image_url }}">
    <meta name="twitter:image:alt" content="{{ cover_alt_text }}">
    <meta name="twitter:player" content="{{ player_url }}">
    <meta name="twitter:player:stream" content="{{ episode.podcast_audio.m4a.url }}">
    <meta name="twitter:player:stream:content_type" content="audio/mp4">
    <meta name="twitter:player:width" content="480">
    <meta name="twitter:player:height" content="240">

    <meta property="og:url" content="{{ page_url }}">
    <meta property="og:title" content="{{ page.seo_title }}">
    <meta property="og:image" content="{{ cover_image_url }}">
    <meta property="og:image:alt" content="{{ cover_alt_text }}">
    <meta property="og:type" content="article">
    <meta property="og:description" content="{{ page.search_description }}">
    <meta property="og:updated_time" content="{{ updated_timestamp }}">
    <meta property="og:audio" content="{{ episode.podcast_audio.m4a.url }}">
    <!--
      todo
      <meta name="twitter:site" content="@PodcastPython">
    -->
  {% else %}
    {{ block.super }}
  {% endif %}
{% endblock %}
