{% load static %}
{% if page.pk %}
  {% if not render_for_feed %}
    <podlove-player
      id="audio_{{ value.pk }}"
      data-variant="xl"
      data-url="{% url 'cast:api:audio_podlove_detail' pk=value.pk post_id=page.pk %}"
      data-embed="{% static 'cast/js/web-player/embed.5.js' %}"
      data-config="{% url 'cast:api:player_config' %}"
    >
    </podlove-player>
  {% endif %}
{% endif %}
