# Generated by Django 5.0.3 on 2024-03-13 22:09

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0054_alter_blog_template_base_dir_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="podcast",
            name="itunes_artwork",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="podcasts",
                to="cast.itunesartwork",
            ),
        ),
    ]
