# Generated by Django 5.0.4 on 2024-05-08 20:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0056_add_cover_image_for_post"),
    ]

    operations = [
        migrations.RenameField(
            model_name="post",
            old_name="cover",
            new_name="cover_image",
        ),
        migrations.AddField(
            model_name="post",
            name="cover_alt_text",
            field=models.CharField(blank=True, default="", max_length=255),
        ),
    ]
